"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var CALC_REG = /\bcalc\(([\s\S]+)\)/;
var PERCENT = /[\d.]+%/;
var VIEWPORT_WIDTH = /[\d.]+vw/;
var VIEWPORT_HEIGHT = /[\d.]+vh/;
var VIEWPORT_MIN = /[\d.]+vmin/;
var VIEWPORT_MAX = /[\d.]+vmax/;
var PIXEL = /(\d+)px/g;
var REM = /[\d.]+rem/;
var MATH_EXP = /[+\-/*]?[\d.]+(px|%|rem|vw|vh|vmin|vmax)?/g;
var PLACEHOLDER = "$1";
var ONLYNUMBERS = /[\s\-0-9]/g;
var PLUS_MINUS = /[-+]/g;
var CALC_WITH_OPERATOR = /^calc\([-+]/;
var MINUS_PERCENTAGE = /\s+\-\d+%/g;
var PLUS_MINUS_WHITESPACE = /\s+(\+|\-)\s+/g;
var DIVIDE_BY_ZERO = /\/\s?0/g;
var MULTIPLY_BY_UNIT = /\d+(px|%|rem|vw|vh|vmin|vmax)\s?\*\s?\d+(px|%|rem|vw|vh|vmin|vmax)/g;
var DIVIDE_BY_UNIT = /\/\s?\d+(px|%|rem|vw|vh|vmin|vmax)/;
var UNITLESS_VALUE_LEFT = /\d+\s+(\+|\-)\s+\d+(px|%|rem|vw|vh|vmin|vmax)/g;
var UNITLESS_VALUE_RIGHT = /\d+(px|%|rem|vw|vh|vmin|vmax)\s+(\+|\-)\s+\d+(\s+|$|\))/g;
var CSS_CALC = "CSS calc(): ";

var noClamp = ["top", "left", "bottom", "right", "z-index", "marginTop", "marginRight", "marginBottom", "marginLeft"];

var transform = exports.transform = function transform(_ref) {
  var prop = _ref.prop,
      value = _ref.value,
      win = _ref.win,
      parent = _ref.parent,
      font = _ref.font;

  var calcMatches = value.match(CALC_REG);
  if (!calcMatches) return;

  if (value.match(UNITLESS_VALUE_LEFT) || value.match(UNITLESS_VALUE_RIGHT)) {
    throw new Error(CSS_CALC + "unexpected unitless value.");
  }

  if (value.match(MULTIPLY_BY_UNIT)) {
    throw new Error(CSS_CALC + ("cannot multiply by \"" + RegExp.$2 + "\", number expected."));
  }

  if (value.match(DIVIDE_BY_UNIT)) {
    throw new Error(CSS_CALC + ("cannot divide by \"" + RegExp.$1 + "\", number expected."));
  }

  if (value.match(DIVIDE_BY_ZERO)) {
    throw new Error(CSS_CALC + "cannot divide by zero.");
  }

  var plusMinus = value.match(PLUS_MINUS);

  if (!value.match(CALC_WITH_OPERATOR) && plusMinus) {
    var minusPercentage = value.match(MINUS_PERCENTAGE);

    if (!minusPercentage) {
      var white = value.match(PLUS_MINUS_WHITESPACE);
      if (plusMinus && !white || plusMinus && white && plusMinus.length !== white.length) {
        throw new Error(CSS_CALC + "white space is required on both sides of the + and - operators.");
      }
    }
  }

  var calcPart = calcMatches[0];
  var formula = calcMatches[1].replace(/calc\(/g, "(");

  var matches = formula.match(MATH_EXP);

  var currentFormula = formula.replace(PIXEL, PLACEHOLDER);

  matches.forEach(function (match) {
    var refValue = void 0;
    var modifier = void 0;

    if (match.match(PERCENT)) {
      if (prop === "fontSize") {
        if (font && font.size !== undefined) {
          refValue = font.size;
        } else {
          throw new Error(CSS_CALC + "unable to calculate font-size.");
        }
      } else {
        refValue = prop === "height" ? parent.height : parent.width;
      }
      modifier = parseFloat(match) / 100;
    } else if (match.match(VIEWPORT_WIDTH)) {
      refValue = win.width;
      modifier = parseFloat(match) / 100;
    } else if (match.match(VIEWPORT_HEIGHT)) {
      refValue = win.height;
      modifier = parseFloat(match) / 100;
    } else if (match.match(VIEWPORT_MIN)) {
      refValue = Math.min(win.width, win.height);
      modifier = parseFloat(match) / 100;
    } else if (match.match(VIEWPORT_MAX)) {
      refValue = Math.max(win.width, win.height);
      modifier = parseFloat(match) / 100;
    } else if (match.match(REM)) {
      refValue = 16;
      modifier = parseFloat(match);
    }

    if (modifier !== undefined) {
      currentFormula = currentFormula.replace(match, refValue * modifier);
    }
  });

  if (currentFormula.match(ONLYNUMBERS)) {
    var result = eval("(" + currentFormula + ")");
    var resultFloat = parseFloat(value.replace(calcPart, result));

    if (noClamp.indexOf(prop) === -1 && resultFloat < 0) {
      return 0;
    }
    return resultFloat;
  }
};